﻿using UnityEngine;
using System.Collections;
using UnityEngine.UI;

public class ClickMaintainer : Singleton<ClickMaintainer>
{
    public GameObject cityNameLabel;
    public GameObject cityNameCanvas;
    private GameObject leftClickedObject;
    private GameObject hoveredObject;
    private RaycastHit2D frontmostRaycastHit;

    private SpriteRenderer spriteRenderer;

    private RaycastHit2D nullRaycast = new RaycastHit2D();

    // Update is called once per frame
    void Update()
    {
        frontmostRaycastHit = GetFrontmostRaycastHit();

        // doing: onMouseHover and onMouseHoverOff
        // RaycastHit2D struct can't == null, so I created empty raycastHit2D for checking if there's no return
        if (frontmostRaycastHit != nullRaycast)
        {
            if (frontmostRaycastHit.collider.gameObject != hoveredObject)
            {
                if (hoveredObject)
                {   // Reset last hover, if able
                    hoveredObject.SendMessage("OnMouseHoverOff", frontmostRaycastHit, SendMessageOptions.DontRequireReceiver);
                }
                hoveredObject = frontmostRaycastHit.collider.gameObject;
                hoveredObject.SendMessage("OnMouseHover", frontmostRaycastHit, SendMessageOptions.DontRequireReceiver);
            }
        }
        else if (hoveredObject)
        {   // Reset last hover, if able
            hoveredObject.SendMessage("OnMouseHoverOff", frontmostRaycastHit, SendMessageOptions.DontRequireReceiver);
            hoveredObject = null;
        }

        // doing: onLeftClick
        if (Input.GetMouseButtonDown(0))
        {

            // RaycastHit2D struct can't == null, so I created empty raycastHit2D for checking if there's no return
            if (hoveredObject)
            {
                leftClickedObject = hoveredObject;
                leftClickedObject.SendMessage("OnLeftClick", frontmostRaycastHit, SendMessageOptions.DontRequireReceiver);
            }
        }

        // doing: onLeftRelease
        if (Input.GetMouseButtonUp(0))
        {
            if (leftClickedObject)
            {
                leftClickedObject.SendMessage("OnLeftRelease", frontmostRaycastHit, SendMessageOptions.DontRequireReceiver);
                leftClickedObject = null;
            }
        }

    }

    RaycastHit2D GetFrontmostRaycastHit()
    {
        Vector3 clickPosition = Camera.main.ScreenToWorldPoint(Input.mousePosition);
        RaycastHit2D[] hits = Physics2D.LinecastAll(clickPosition, clickPosition);

        if (hits.Length != 0)
        {
            int topSortingLayer = 0;
            int[] sortingLayerIDArray = new int[hits.Length];
            int[] sortingOrderArray = new int[hits.Length];
            int topSortingOrder = 0;
            int indexOfTopSortingOrder = 0;

            for (var i = 0; i < hits.Length; i++)
            {
                spriteRenderer = hits[i].collider.gameObject.GetComponent<SpriteRenderer>();

                sortingLayerIDArray[i] = spriteRenderer.sortingLayerID;

                sortingOrderArray[i] = spriteRenderer.sortingOrder;
            }

            for (var j = 0; j < sortingLayerIDArray.Length; j++)
            {
                if (sortingLayerIDArray[j] >= topSortingLayer)
                {
                    topSortingLayer = sortingLayerIDArray[j];
                }
            }

            for (var k = 0; k < sortingOrderArray.Length; k++)
            {
                if (sortingOrderArray[k] >= topSortingOrder && sortingLayerIDArray[k] == topSortingLayer)
                {
                    topSortingOrder = sortingOrderArray[k];
                    indexOfTopSortingOrder = k;
                }
                else
                {
                    // Do nothing and continue loop.
                }
            }
            return hits[indexOfTopSortingOrder];
        }
        else 
        {
            return nullRaycast;
        }
    }

    public void ChangeCityLabel(Vector3 position, string name)
    {
        float cameraScale = Camera.main.orthographicSize;
        float canvasScale = (cameraScale /18) * (0.3f / 7);
        cityNameCanvas.transform.position = position + new Vector3(0, 1);
        cityNameCanvas.transform.localScale = new Vector3(canvasScale, canvasScale);
        cityNameCanvas.SetActive(true);
        cityNameLabel.GetComponent<Text>().text = name;
    }

    public void HideCityLabel()
    {
        cityNameCanvas.SetActive(false);
    }
}