﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using UnityEngine.UI;


public class InCityManager : Singleton<InCityManager> {

    public enum InCityErrors { WagonNotEmpty, NoSpaceForCoal, NoSpaceForGoods, NoCurrency, NoGoods, NoSpaceForWagon }

    public GameObject CityNameGO;
    public GameObject SingleGoodsGO;
    public GameObject SingleBuyWagonGO;
    public GameObject SingleMyWagonGO;
    public GameObject[] subPanels;
    public GameObject errorTxtGO;
    public GameObject coalNumberGO;

    public List<GameObject> createdPanels = new List<GameObject>();
    CityPrefab currentCity = null;

	// Use this for initialization
	void Start () {
	
	}
	
	// Update is called once per frame
	void Update () {
	
	}

    public void EnterCity(CityPrefab city)
    {
        if (city == null) Debug.Log("crash");
        currentCity = city;
        CityNameGO.GetComponent<Text>().text = city.CityName;
        DrawGoods();
        errorTxtGO.SetActive(false);
    }

    public void DrawGoods()
    {
        DestroyCreatedPanels();
        List<ThingAndPrice> things = currentCity.ThingsForSell;
        int counter = 0;
        foreach (ThingAndPrice loopThing in things)
        {
            if (loopThing.IsWagon) continue;

            GameObject newWagonPanel = (GameObject)GameObject.Instantiate(SingleGoodsGO);
            newWagonPanel.transform.SetParent(subPanels[2].transform);
            newWagonPanel.transform.localPosition = Vector3.zero;
            newWagonPanel.transform.Translate(new Vector3(0, (counter++) * -70 + 60));
            newWagonPanel.GetComponent<SingleGoods>().Config(loopThing);

            createdPanels.Add(newWagonPanel);
        }
    }

    public void DrawBuyWagons()
    {
        DestroyCreatedPanels();
        List<ThingAndPrice> things = currentCity.ThingsForSell;
        int counter = 0;
        foreach (ThingAndPrice loopThing in things)
        {
            if (!loopThing.IsWagon) continue;

            GameObject newWagonPanel = (GameObject)GameObject.Instantiate(SingleBuyWagonGO);
            newWagonPanel.transform.SetParent(subPanels[2].transform);
            newWagonPanel.transform.localPosition = Vector3.zero;
            newWagonPanel.transform.Translate(new Vector3(0, (counter++) * -40 + 70));
            newWagonPanel.GetComponent<SingleBuyWagon>().Config(loopThing);

            createdPanels.Add(newWagonPanel);
        }
    }

    public void DrawMyWagons()
    {
        DestroyCreatedPanels();
        List<Wagon> wagons = TrainController.Instance.GetWagons();
        int counter = 0;
        foreach (Wagon loopWagon in wagons)
        {
            GameObject newWagonPanel = (GameObject)GameObject.Instantiate(SingleMyWagonGO);
            newWagonPanel.transform.SetParent(subPanels[3].transform);
            newWagonPanel.transform.localPosition = Vector3.zero;
            newWagonPanel.transform.Translate(new Vector3(0, (counter++) * -40 + 70));
            newWagonPanel.GetComponent<SingleMyWagon>().Config(loopWagon, CheckIfCitySellWagon(loopWagon.typeRes));

            createdPanels.Add(newWagonPanel);
        }
    }

    private bool CheckIfCitySellWagon(ResourcesTypes wagonType)
    {
        foreach (ThingAndPrice thing in currentCity.ThingsForSell)
        {
            if (thing.IsWagon && thing.TypeOfRes == wagonType) return true;
        }
        return false;
    }

    private void DestroyCreatedPanels()
    {
        foreach (GameObject loop in createdPanels)
        {
            GameObject.Destroy(loop);
        }
        createdPanels.Clear();

        coalNumberGO.GetComponent<Text>().text = TrainController.Instance.GetResourceAmmount(ResourcesTypes.Coal).ToString();
        TrainController.Instance.RefreshStats();
    }

    public void ReportError(InCityErrors errorType)
    {
        string errorMsg;
        switch (errorType)
        {
            case InCityErrors.WagonNotEmpty: errorMsg = "Cannot sell non-empty wagon"; break;
            case InCityErrors.NoSpaceForCoal: errorMsg = "You don't have space for currency"; break;
            case InCityErrors.NoSpaceForGoods: errorMsg = "You don't have space for goods"; break;
            case InCityErrors.NoSpaceForWagon: errorMsg = "You can't have more wagons"; break;
            case InCityErrors.NoCurrency: errorMsg = "You don't have enough coal"; break;
            case InCityErrors.NoGoods: errorMsg = "You don't have enough goods"; break;
            default: errorMsg = "Unknown error"; break;
        }
        errorTxtGO.SetActive(true);
        errorTxtGO.GetComponent<Text>().text = errorMsg;
    }

    public void BuyGoods(ThingAndPrice thing, int quantity)
    {
        if (TrainController.Instance.GetSpaceForResource(thing.TypeOfRes) >= quantity)
        {
            int goodsPrice = currentCity.ThingsForSell.Find(x => x.TypeOfRes == thing.TypeOfRes && !x.IsWagon).Price * quantity;
            if (TrainController.Instance.GetResourceAmmount(ResourcesTypes.Coal) >= goodsPrice)
            {
                TrainController.Instance.AddRemoveResource(ResourcesTypes.Coal, goodsPrice * -1);
                TrainController.Instance.AddRemoveResource(thing.TypeOfRes, quantity);
                InCityManager.Instance.DrawGoods();
                errorTxtGO.SetActive(false);
                SoundMaintainer.Instance.PlaySound(Sounds.Money);
            }
            else
            {
                InCityManager.Instance.ReportError(InCityManager.InCityErrors.NoCurrency);
            }
        }
        else
        {
            InCityManager.Instance.ReportError(InCityManager.InCityErrors.NoSpaceForGoods);
        }
    }

    public void SellGoods(ThingAndPrice thing, int quantity)
    {
        int goodsPrice = currentCity.ThingsForSell.Find(x => x.TypeOfRes == thing.TypeOfRes && !x.IsWagon).Price * quantity;
        if (TrainController.Instance.GetSpaceForResource(ResourcesTypes.Coal) >= goodsPrice)
        {
            if (TrainController.Instance.GetResourceAmmount(thing.TypeOfRes) >= quantity)
            {
                TrainController.Instance.AddRemoveResource(ResourcesTypes.Coal, goodsPrice);
                TrainController.Instance.AddRemoveResource(thing.TypeOfRes, quantity * -1);
                InCityManager.Instance.DrawGoods();
                errorTxtGO.SetActive(false);
                SoundMaintainer.Instance.PlaySound(Sounds.Money);
            }
            else
            {
                InCityManager.Instance.ReportError(InCityManager.InCityErrors.NoGoods);
            }
        }
        else
        {
            InCityManager.Instance.ReportError(InCityManager.InCityErrors.NoSpaceForCoal);
        }
    }

    public void BuyWagon(ThingAndPrice wagon, GameObject wagonPrefab)
    {
        if (TrainController.Instance.HasSpaceForAnotherWagon())
        {
            int wagonPrice = currentCity.ThingsForSell.Find(x => x.TypeOfRes == wagon.TypeOfRes && x.IsWagon).Price;
            if (TrainController.Instance.GetResourceAmmount(ResourcesTypes.Coal) >= wagonPrice)
            {
                TrainController.Instance.AddRemoveResource(ResourcesTypes.Coal, wagonPrice * -1);
                GameObject newWagon = (GameObject)GameObject.Instantiate(wagonPrefab);
                newWagon.transform.parent = TrainController.Instance.transform;
                TrainController.Instance.AddWagon(newWagon.GetComponent<Wagon>());
                InCityManager.Instance.DrawBuyWagons();
                errorTxtGO.SetActive(false);
                SoundMaintainer.Instance.PlaySound(Sounds.Money);
            }
            else
            {
                InCityManager.Instance.ReportError(InCityManager.InCityErrors.NoCurrency);
            }
        }
        else
        {
            InCityManager.Instance.ReportError(InCityManager.InCityErrors.NoSpaceForWagon);
        }
    }

    public void SellWagon(Wagon wagon)
    {
        if (wagon.currentCapacity == 0)
        {
            int wagonPrice = currentCity.ThingsForSell.Find(x => x.TypeOfRes == wagon.typeRes && x.IsWagon).Price;
            wagonPrice = wagonPrice * 3 / 4;
            if (TrainController.Instance.AddRemoveResource(ResourcesTypes.Coal, wagonPrice, wagon))
            {
                TrainController.Instance.DestroyWagon(wagon);
                InCityManager.Instance.DrawMyWagons();
                errorTxtGO.SetActive(false);
                SoundMaintainer.Instance.PlaySound(Sounds.Money);
            }
            else 
            {
                InCityManager.Instance.ReportError(InCityManager.InCityErrors.NoSpaceForCoal);
            }
        }
        else
        {
            InCityManager.Instance.ReportError(InCityManager.InCityErrors.WagonNotEmpty);
        }
    }
}
