﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class MapMaintainer : Singleton<MapMaintainer> {

    public TextAsset mapSource;
    public TextAsset cityNamesSource;
    public GameObject railPrefab;
    public GameObject cityPrefab;
    public GameObject[] decorationPrefabs;
    public GameObject railFolder;

    const int MAP_WIDTH = 128;
    const int MAP_HEIGHT = 72;
    const int FIRST_CITY = 13;

    private List<RailField> railFields;
    private List<CityPrefab> cities;
    private List<string> cityNames;

	// Use this for initialization
	void Start () {
        LoadMap();
	}
	
	// Update is called once per frame
	void Update () {
	}

    void LoadMap()
    {
        railFields = new List<RailField>();
        cities = new List<CityPrefab>();
        cityNames = new List<string>(cityNamesSource.text.Split("\n"[0]));
        Shuffle(cityNames);

        string mapData = mapSource.text.Substring(mapSource.text.IndexOf("data=")+7, MAP_HEIGHT * (MAP_WIDTH * 2 + 2));
        int currentX = 0;
        int currentY = 0;
        foreach (char loopChar in mapData)
        {
            if (loopChar >= 48 && loopChar <= 57)
            {
                if ((++currentX) > 128)
                {
                    currentX -= 128;
                    currentY += 1;
                }
                if (loopChar >= 49)
                {
                    LoadField(currentX, currentY, loopChar - 48);
                }
            }
        }

        foreach (RailField loopRailField in railFields)
        {
            loopRailField.CheckIfSwitchable();
        }

        foreach (CityPrefab loopCity in cities)
        {
            loopCity.FindOutDirections();
        }

        TrainController.Instance.SetStartingCity(GetStartingCity());
    }

    void LoadField(int x, int y, int charNumber)
    {
        if (charNumber == 8)
        {
            int decoIndex = Random.Range(0, 2);
            GameObject newDecoGO = (GameObject)GameObject.Instantiate(decorationPrefabs[decoIndex], new Vector3(x,-1*y), Quaternion.Euler(new Vector3(0,0,Random.Range(0f,180f))));
            newDecoGO.transform.parent = railFolder.transform;
        }
        else if (charNumber == 7)
        {
            //miasto
            GameObject newCityGO = (GameObject)GameObject.Instantiate(cityPrefab);
            newCityGO.transform.parent = railFolder.transform;
            newCityGO.GetComponent<CityPrefab>().Config(x, y, cityNames[cities.Count]);
            cities.Add(newCityGO.GetComponent<CityPrefab>());
        }
        else
            //tor
        {
            GameObject newRailGO = (GameObject)GameObject.Instantiate(railPrefab);
            newRailGO.transform.parent = railFolder.transform;

            RailsTypes newType = RailsTypes.V;
            switch (charNumber)
            {
                case 1: newType = RailsTypes.V; break;
                case 2: newType = RailsTypes.H; break;
                case 3: newType = RailsTypes.Q1; break;
                case 4: newType = RailsTypes.Q2; break;
                case 5: newType = RailsTypes.Q3; break;
                case 6: newType = RailsTypes.Q4; break;
                default: newType = RailsTypes.V; break;
            }

            RailField newRailField = new RailField(x, y, newType, newRailGO);
            railFields.Add(newRailField);
            newRailGO.GetComponent<RailPrefab>().Config(x, y, newType, newRailField);
        }
    }

    public CityPrefab GetStartingCity()
    {
        return cities[FIRST_CITY];
    }

    public RailField FindField(int x, int y)
    {
        return railFields.Find(field => field.X == x && field.Y == y);
    }

    public static void Shuffle(List<string> list)
    {
        int n = list.Count;
        while (n > 1)
        {
            n--;
            int k = Random.Range(0, n + 1);
            string value = list[k];
            list[k] = list[n];
            list[n] = value;
        }
    }

}
