﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

enum Directions { Up, Right, Down, Left }

public class RailField
{

    public int X { get; private set; }
    public int Y { get; private set; }

    public Vector3 Position { get; private set; }
    public RailsTypes type { get; private set; }
    public bool Switchable { get; private set; }

    private List<RailsTypes> possibleTypes;
    private int currentTypeIndex = 0;
    private GameObject prefab;

    public RailField(int x, int y, RailsTypes type, GameObject prefab)
    {
        this.X = x;
        this.Y = y;
        Position = new Vector3(x, -1 * y);
        this.type = type;
        this.prefab = prefab;
    }

    public RailField GetNextField(WagonDirection direction)
    {
        switch (type)
        {
            case RailsTypes.H:
                if (direction == WagonDirection.TopRight)
                    return FindField(Directions.Right);
                else
                    return FindField(Directions.Left);
            case RailsTypes.V:
                if (direction == WagonDirection.TopRight)
                    return FindField(Directions.Up);
                else
                    return FindField(Directions.Down);
            case RailsTypes.Q1:
                if (direction == WagonDirection.TopRight)
                    return FindField(Directions.Left);
                else
                    return FindField(Directions.Down);
            case RailsTypes.Q2:
                if (direction == WagonDirection.TopRight)
                    return FindField(Directions.Up);
                else
                    return FindField(Directions.Left);
            case RailsTypes.Q3:
                if (direction == WagonDirection.TopRight)
                    return FindField(Directions.Up);
                else
                    return FindField(Directions.Right);
            case RailsTypes.Q4:
                if (direction == WagonDirection.TopRight)
                    return FindField(Directions.Right);
                else
                    return FindField(Directions.Down);
        }
        return null;
    }

    private RailField FindField(Directions dir)
    {
        switch (dir)
        {
            case Directions.Up: return MapMaintainer.Instance.FindField(X, Y - 1);
            case Directions.Right: return MapMaintainer.Instance.FindField(X + 1, Y);
            case Directions.Down: return MapMaintainer.Instance.FindField(X, Y + 1);
            case Directions.Left: return MapMaintainer.Instance.FindField(X - 1, Y);
        }
        return null;
    }

    public void CheckIfSwitchable()
    {
        if (type == RailsTypes.H || type == RailsTypes.V)
        {
            Switchable = false;
            return;
        }
        possibleTypes = new List<RailsTypes>();
        if (FindField(Directions.Left) != null && FindField(Directions.Right) != null) possibleTypes.Add(RailsTypes.H);
        if (FindField(Directions.Up) != null && FindField(Directions.Down) != null) possibleTypes.Add(RailsTypes.V);
        if (FindField(Directions.Left) != null && FindField(Directions.Down) != null) possibleTypes.Add(RailsTypes.Q1);
        if (FindField(Directions.Up) != null && FindField(Directions.Left) != null) possibleTypes.Add(RailsTypes.Q2);
        if (FindField(Directions.Right) != null && FindField(Directions.Up) != null) possibleTypes.Add(RailsTypes.Q3);
        if (FindField(Directions.Down) != null && FindField(Directions.Right) != null) possibleTypes.Add(RailsTypes.Q4);
        if (possibleTypes.Count > 1)
        {
            Switchable = true;
            currentTypeIndex = FindCurrentTypeIndex();
        }
        else Switchable = false;
    }

    private int FindCurrentTypeIndex()
    {
        for (int loop = 0; loop < possibleTypes.Count; loop++)
        {
            if (possibleTypes[loop] == type) return loop;
        }
        return 0;
    }

    public void Switch()
    {
        if (!Switchable) return;
        if (++currentTypeIndex >= possibleTypes.Count) currentTypeIndex = 0;
        type = possibleTypes[currentTypeIndex];
        prefab.GetComponent<RailPrefab>().ChangeType(type);
    }

}
