﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using UnityEngine.UI;

public class TrainController : Singleton<TrainController> {

    public bool IsInCity { get { if (inCity != null) return true; return false; } }
    public GameObject cityPanel;
    public GameObject gameOverPanel;
    public GameObject statCoalGO;
    public GameObject statDistanceGO;
    public GameObject fogPrefab;

    private List<Wagon> wagons;
    public float maxSpeed;
    public float startSpeed;
    private float speed;
    private int distanceMeter = 0;

    private bool beforeFirstStart = true;

    private CityPrefab inCity;

	// Use this for initialization
	void Start () {
        wagons = new List<Wagon>(transform.GetComponentsInChildren<Wagon>());
	}
	
	// Update is called once per frame
	void Update () {

        if (Input.GetKeyDown(KeyCode.R))
        {
            Application.LoadLevel(0);
        }

        if (!IsInCity)
        {
            if (Input.GetKeyDown(KeyCode.Q))
            {
                ChangeSpeed(speed + 0.5f);
            }

            if (Input.GetKeyDown(KeyCode.A))
            {
                ChangeSpeed(speed - 0.5f);
            }

            if (Input.GetKeyDown(KeyCode.Space))
            {
                SoundMaintainer.Instance.PlaySound(Sounds.ChooChoo);
            }
        }
        if (beforeFirstStart) return;

        if (CheckIfFullyInCity())
        {
            if (!IsInCity)
            {
                inCity = wagons[0].InCity;
                JustEnteredCity();
            }
            else
            {
                inCity = wagons[0].InCity;
            }
        }
        if (CheckIfFullyOutsideCity())
        {
            inCity = null;
        }

	}

    private bool CheckIfFullyInCity()
    {
        foreach (Wagon loopWagon in wagons)
        {
            if (!loopWagon.IsInCity) return false;
        }
        return true;
    }

    private bool CheckIfFullyOutsideCity()
    {
        foreach (Wagon loopWagon in wagons)
        {
            if (loopWagon.IsInCity) return false;
        }
        return true;
    }

    private void ChangeSpeed(float newSpeed)
    {
        speed = newSpeed;
        if (speed > maxSpeed) speed = maxSpeed;
        if (speed <= 0) speed = 0;
        foreach (Wagon loopWagon in wagons)
        {
            loopWagon.ChangeSpeed(speed);
        }
    }

    private IEnumerator StartWagons()
    {
        speed = startSpeed;
        float time = 2f / speed;
        foreach (Wagon loopWagon in wagons)
        {
            loopWagon.StartWagon(inCity.GetOutField,inCity.GetOutDirection);
            loopWagon.ChangeSpeed(speed);
            yield return new WaitForSeconds(time);
        }
    }

    private void JustEnteredCity()
    {
        cityPanel.SetActive(true);
        InCityManager.Instance.EnterCity(inCity);
    }

    public void TakeOffTrain()
    {
        if (IsInCity)
        {
            StartCoroutine(StartWagons());
            cityPanel.SetActive(false);
            beforeFirstStart = false;
        }
    }

    public void SetStartingCity(CityPrefab city)
    {
        inCity = city;
        foreach (Wagon loopWagon in wagons)
        {
            loopWagon.transform.position = inCity.transform.position;
        }
        Camera.main.GetComponent<CameraMaintainer>().PrepareForStart(wagons[0]);

        InCityManager.Instance.EnterCity(inCity);
        RefreshStats();
    }

    private void RefreshStartingStuff()
    {
        foreach (Wagon loopWagon in wagons)
        {
            loopWagon.transform.position = inCity.transform.position;
        }
    }

    public List<Wagon> GetWagons()
    {
        return wagons.GetRange(1, wagons.Count - 1);
    }

    public void AddWagon(Wagon w)
    {
        wagons.Add(w);
        RefreshStartingStuff();
    }

    public void DestroyWagon(Wagon w)
    {
        w.DestroyMe();
        wagons.Remove(w);
    }

    public bool AddRemoveResource(ResourcesTypes type, int ammount)
    {
        return AddRemoveResource(type, ammount, null);
    }

    public bool AddRemoveResource(ResourcesTypes type, int ammount, Wagon wagonToNotCheck)
    {
        if (ammount < 0)
        {
            ammount *= -1;
            if (GetResourceAmmount(type) < ammount) return false;
            foreach (Wagon loopWagon in wagons)
            {
                if (loopWagon.typeRes == type)
                {
                    if (loopWagon.currentCapacity >= ammount)
                    {
                        loopWagon.currentCapacity -= ammount;
                        ammount = 0;
                    }
                    else
                    {
                        ammount -= loopWagon.currentCapacity;
                        loopWagon.currentCapacity = 0;
                    }
                    if (ammount <= 0) return true;
                }
            }
        }
        else
        {
            if (GetSpaceForResource(type,wagonToNotCheck) < ammount) return false;
            foreach (Wagon loopWagon in wagons)
            {
                if (loopWagon.typeRes == type)
                {
                    int spaceHere = loopWagon.maxCapacity - loopWagon.currentCapacity;
                    if (ammount <= spaceHere)
                    {
                        loopWagon.currentCapacity += ammount;
                        ammount = 0;
                    }
                    else
                    {
                        loopWagon.currentCapacity += spaceHere;
                        ammount -= spaceHere;
                    }
                    if (ammount <= 0) return true;
                }
            }
        }
        return false;
    }

    public int GetResourceAmmount(ResourcesTypes type)
    {
        int summaryAmmount = 0;
        foreach (Wagon loopWagon in wagons)
        {
            if (loopWagon.typeRes == type) summaryAmmount += loopWagon.currentCapacity;
        }
        return summaryAmmount;
    }

    public int GetSpaceForResource(ResourcesTypes type)
    {
        return GetSpaceForResource(type, null);
    }

    public int GetSpaceForResource(ResourcesTypes type, Wagon wagonToNotCheck)
    {
        int spaceForCoal = 0;
        foreach (Wagon loopWagon in wagons)
        {
            if (loopWagon.typeRes == type && loopWagon != wagonToNotCheck)
            {
                spaceForCoal += (loopWagon.maxCapacity - loopWagon.currentCapacity);
            }
        }
        return spaceForCoal;
    }

    public bool HasSpaceForAnotherWagon()
    {
        if (wagons.Count < 6) return true;
        return false;
    }

    public void ConsumeCoalTick()
    {
        if (!AddRemoveResource(ResourcesTypes.Coal, -1))
        {
            BlockTrain();
        }
        else
        {
            SoundMaintainer.Instance.PlaySound(Sounds.Chaga);
            distanceMeter++;
            RefreshStats();
            GameObject.Instantiate(fogPrefab,wagons[0].transform.position,new Quaternion());
        }
    }

    public void RefreshStats()
    {
        statCoalGO.GetComponent<Text>().text = GetResourceAmmount(ResourcesTypes.Coal).ToString();
        statDistanceGO.GetComponent<Text>().text = distanceMeter.ToString() + "km";
    }

    private void BlockTrain()
    {
        ChangeSpeed(0);
        maxSpeed = 0;
        gameOverPanel.SetActive(true);
    }
}
