﻿using UnityEngine;
using System.Collections;

public enum WagonDirection { TopRight, BottomLeft }

public class Wagon : MonoBehaviour
{
    public ResourcesTypes typeRes;
    public int currentCapacity;
    public int maxCapacity;
    public bool isTrain;

    private float speed;

    private WagonDirection direction = WagonDirection.TopRight;
    private RailField currentRailField;
    private RailField nextRailField;

    //private float previousAngle = 0;
    //private float nextAngle = 0;

    public bool IsInCity { get { if (InCity != null) return true; return false; } }
    public CityPrefab InCity { get; private set; }

    private const int FIELDS_PER_CONSUMPTION = 5;
    private int fieldsToConsumeCoalLeft = FIELDS_PER_CONSUMPTION;

    // Use this for initialization
    void Start()
    {
    }

    // Update is called once per frame
    void Update()
    {
        if (Input.GetKeyDown(KeyCode.L))
        {
            InvertDirection();
            LookForNextDestination();
        }

        if (speed != 0 && nextRailField != null)
        {
            float moveStep = speed * Time.deltaTime;
            if (Vector3.Distance(transform.position, nextRailField.Position) < 0.1f)
            {
                bool wasTypeOfH = false;
                if (currentRailField.type == RailsTypes.H) wasTypeOfH = true;

                currentRailField = nextRailField;

                if (currentRailField.type == RailsTypes.Q1 || currentRailField.type == RailsTypes.Q3)
                {
                    if (wasTypeOfH) InvertDirection();
                    LookForNextDestination();
                    if (!wasTypeOfH) InvertDirection();
                }
                else
                {
                    LookForNextDestination();
                }
                if (isTrain)
                {
                    if (fieldsToConsumeCoalLeft-- <= 0)
                    {
                        fieldsToConsumeCoalLeft = FIELDS_PER_CONSUMPTION;
                        TrainController.Instance.ConsumeCoalTick();
                    }
                }
                return;
            }
            transform.position = Vector3.MoveTowards(transform.position, nextRailField.Position, moveStep);
            //float angle = Mathf.LerpAngle(previousAngle, nextAngle, Vector3.Distance(transform.position,nextRailField.Position));
            //Debug.Log(previousAngle + " / " + nextAngle + " / " + Vector3.Distance(transform.position, nextRailField.Position));
            //transform.eulerAngles = new Vector3(0, 0, angle);
            RotateWagon(nextRailField.Position - transform.position);
        }
    }

    private void RotateWagon(Vector3 moveVector)
    {
        if (moveVector.x > 0.1) transform.eulerAngles = new Vector3(0, 0, 180);
        else if (moveVector.x < -0.1) transform.eulerAngles = new Vector3(0, 0, 0);
        else if (moveVector.y > 0.1) transform.eulerAngles = new Vector3(0, 0, -90);
        else if (moveVector.y < -0.1) transform.eulerAngles = new Vector3(0, 0, 90);
    }

    private void InvertDirection()
    {
        if (direction == WagonDirection.TopRight) direction = WagonDirection.BottomLeft;
        else direction = WagonDirection.TopRight;
    }

    private void LookForNextDestination()
    {
        nextRailField = currentRailField.GetNextField(direction);
        //previousAngle = nextAngle;
        //nextAngle = GetEulerAngleByRailsType(currentRailField.type);
   //     transform.rotation = Quaternion.Euler(new Vector3(0, 0, GetEulerAngleByRailsType(currentRailField.type)));

    }

    private float GetEulerAngleByRailsType(RailsTypes type)
    {
        switch (type)
        {
            case RailsTypes.H: return 0;
            case RailsTypes.V: return 90;
            case RailsTypes.Q1: return -45;
            case RailsTypes.Q2: return 45;
            case RailsTypes.Q3: return -45;
            case RailsTypes.Q4: return 45;
            default: return 0;
        }
    }

    public void StartWagon(RailField newField,WagonDirection newDirection)
    {
        this.direction = newDirection;
        currentRailField = newField;
        transform.position = currentRailField.Position;
        LookForNextDestination();
    }

    public void ChangeSpeed(float newSpeed)
    {
        speed = newSpeed;
    }

    private void StopWagon()
    {
        speed = 0;
    }

    public void DestroyMe()
    {
        GameObject.Destroy(gameObject);
    }

    void OnTriggerEnter2D(Collider2D other)
    {
        if (other.tag == "City")
        {
            if (other.GetComponent<CityPrefab>() != null)
            {
                InCity = other.GetComponent<CityPrefab>();
            }
        }
    }

    void OnTriggerExit2D(Collider2D other)
    {
        if (other.tag == "City")
        {
            InCity = null;
        }
    }
}
